@@ -4,7 +4,7 @@ import ctypes
 import queue
 import time
 import psutil
-
+from PyQt5.QtWidgets import QMessageBox
 from pyrateshield import labels, Logger, GLOBAL_LOG_LEVEL
 
 from pyrateshield.radtracer import radtracer
@@ -13,7 +13,7 @@ import pandas as pd
 
 LOG_LEVEL = GLOBAL_LOG_LEVEL
 
-
+UNSUPPORTED_PYSHIELD_ISOTOPES = ['Y-90']
 CRITICAL_POINT_NAME         = 'Critical Point Name'
 RADTRACER_DOSE              = 'RadTracer Dose [mSv]'
 PYSHIELD_DOSE               = 'PyShield Dose [mSv]'
@@ -115,21 +115,48 @@ class Dosemapper(Logger):
             
         if project.dosemap.engine == labels.PYSHIELD:
             pyshield_engine = Engine.from_pyrateshield(project)
-        
+         
         for source in sources:
             if not source.enabled:
                 continue
 
-            if source.label == labels.SOURCES_NM and project.dosemap.engine == labels.PYSHIELD:
+            if project.dosemap.engine == labels.PYSHIELD and Dosemapper.is_supported_by_pyshield(source):
                 source_dosemap = pyshield_engine.source_dosemap(source)
             else:
                 source_dosemap = radtracer.dosemap_single_source(source, project)
         
             dosemap = update_dosemap(source_dosemap)
         return dosemap
+    
+    @staticmethod
+    def is_supported_by_pyshield(src):
+        return src.label == labels.SOURCES_NM\
+                and src.isotope not in UNSUPPORTED_PYSHIELD_ISOTOPES
 
+    @staticmethod
+    def get_pyshield_supported_sources(sources):
 
+        sources_supported = [src for src in sources\
+                             if Dosemapper.is_supported_by_pyshield(src)]
+
+        # show an additional warning when unsupported isotopes are present.                             
+        unsupported_isotopes = [src.isotope for src in sources\
+                                if src.label==labels.SOURCES_NM\
+                                and src.isotope in UNSUPPORTED_PYSHIELD_ISOTOPES]
+             
+        unsupported_isotopes = list(set(unsupported_isotopes))
+        
+        if len(unsupported_isotopes) > 0:           
+            print(('The following isotopes are not supported by pyshield. '
+                   'Pyshield does not has a model for bremsstrahlung. '
+                   'Calculations will be performed by Radtracer for '
+                   f'these isotopes.\n Isotopes: {unsupported_isotopes}'))
+            
+        return sources_supported
+
+        
     def get_dosemap(self, project, sources=None):
+
         if not self.multi_cpu:
             return self.get_single_cpu_dosemap(project, sources)
         # Process all sources by default
@@ -139,21 +166,24 @@ class Dosemapper(Logger):
         # Exclude sources which are not set to Enabled
         sources = [src for src in sources if src.enabled]
         
-        
+
         # Check if there is any work to be done
         if not len(sources):
             return None
         
-                
-        #This is a bit of a hack to ensure PyShield only receives NM sources,
-        #and that sources with the same position are processed by the same worker.
-        #This allows PyShield to benefit from caching.
-        sources_non_nm = [src for src in sources if src.label != labels.SOURCES_NM]
+        if project.dosemap.engine == labels.PYSHIELD:
+            pyshield_supported_sources = self.get_pyshield_supported_sources(sources)
+            pyshield_unsupported_sources = [src for src in sources\
+                                            if src not in pyshield_supported_sources]                
+    
+         #This is a bit of a hack to ensure PyShield only receives NM sources,
+         #and that sources with the same position are processed by the same worker.
+         #This allows PyShield to benefit from caching.
+            
         if project.dosemap.engine == labels.PYSHIELD:
             src_pos_dct = {}
-            sources_nm = [src for src in sources if src not in sources_non_nm]
 
-            for source in sources_nm:
+            for source in pyshield_supported_sources:
                 # Make a dictionary of source_positions and group the corresponding sources
                 # I.e.: { (x,y,z): [src1, src2, src3 ], (x,y,z): [src4, src5, ... ] }
                 src_pos_dct.setdefault(tuple(source.position), []).append(source)
@@ -177,11 +207,11 @@ class Dosemapper(Logger):
         for i in range(nr_of_sources):
             dosemap = update_dosemap( self.dosemap_queue.get() )        
         
-        if project.dosemap.engine == labels.PYSHIELD and len(sources_non_nm):
+        if project.dosemap.engine == labels.PYSHIELD and len(pyshield_unsupported_sources):
             # Temporarily switch to RadTracer to process the non-NM sources
-            print(f"Switching to RadTracer for {len(sources_non_nm)} non-NM sources")
+            print(f"Switching to RadTracer for {len(pyshield_unsupported_sources)} non-NM and/or bremsstralhung sources")
             project.dosemap.engine = labels.RADTRACER
-            dosemap = update_dosemap( self.get_dosemap(project, sources=sources_non_nm) )
+            dosemap = update_dosemap( self.get_dosemap(project, sources=pyshield_unsupported_sources) )
             print("Done... Switching back to pyShield")
             project.dosemap.engine = labels.PYSHIELD
         
@@ -237,7 +267,7 @@ class Dosemapper(Logger):
                 dose_radtracer = radtracer.pointdose_single_source(
                     crit_point.position, source, project)
 
-                if source.label == labels.SOURCES_NM:
+                if Dosemapper.is_supported_by_pyshield(source):
                     dose_pyshield = pyshield_engine.dose_at_point(crit_point.position,
                                                          sources=[source])
                 else:
