@@ -125,11 +125,18 @@ class DosemapStyle(model_items.ModelItem):
         ### Fix for backwards compatibility: add the is_active flag to the end
         ### of the contour_line list if it wasn't already present.
         ### #TODO: remove this classmethod after next release.
+
+        # dct is None for older versions of pyshield files
+        
+        if dct is None:
+            return cls()
+        
         for cl in dct[labels.CONTOUR_LINES]:
             if len(cl) == 4:
                 cl.append(True)
-        
         return super().from_dict(dct)
+
+        
             
 
 class Dosemap(model_items.ModelItem):
