@@ -19,11 +19,8 @@ from pyrateshield.gui.item_controllers import (EditSourceXrayController,
                                                EditClearanceModel,
                                                EditPixelSizeController,
                                                EditMaterialsController)
-                                               
-                                               
+                                                                                              
 from pyrateshield import labels
-
-
     
 class MainController():
     _mpl_controller = None
@@ -34,7 +31,7 @@ class MainController():
     def __init__(self, dosemapper=None, model=None, view=None):
         
         if dosemapper is None:
-            dosemapper = Dosemapper(multi_cpu=False)
+            dosemapper = Dosemapper(multi_cpu=True)
         
         if model is None:
             model = Model()
@@ -309,7 +306,7 @@ if __name__ == "__main__":
     
     app = QApplication([])
     
-    controller = MainController(model=None, dosemapper=None)
+    controller = MainController(model=model, dosemapper=None)
     window = controller.view
     window.show()    
     app.exec_()
