@@ -48,28 +48,36 @@ DEFAULTS = Defaults()
 
 
 class Model(Observable):
-    _floorplan = None
-    _selected_item = None
-    filename = None
+    filename            = None
     
-    _sources_nm = None
-    _sources_ct = None
-    _sources_xray = None
-    _walls = None
-    _clearances = None
-    _shieldings = None
-    _critical_points = None
-    _floorplan = None
-    _dosemap = None
-    _dosemap_style = None
-    _materials = None
+    _floorplan          = None
+    _selected_item      = None    
+    _sources_nm         = None
+    _sources_ct         = None
+    _sources_xray       = None
+    _walls              = None
+    _clearances         = None
+    _shieldings         = None
+    _critical_points    = None
+    _floorplan          = None
+    _dosemap            = None
+    _dosemap_style      = None
+    _materials          = None
     
     EVENT_UPDATE_FLOORPLAN = 'event_update_floorplan'    
     
-    def __init__(self, floorplan=None, dosemap=None, dosemap_style=None, 
-                 shieldings=None, walls=None, critical_points=None, 
-                 sources_ct=None, sources_nm=None, sources_xray=None,
-                 clearances=None, materials=None):  
+    def __init__(self, 
+                 floorplan          =None, 
+                 dosemap            =None, 
+                 dosemap_style      =None, 
+                 shieldings         =None, 
+                 walls              =None, 
+                 critical_points    =None, 
+                 sources_ct         =None, 
+                 sources_nm         =None, 
+                 sources_xray       =None,
+                 clearances         =None, 
+                 materials          =None):  
         
 
         Observable.__init__(self, log_level=LOG_LEVEL)
@@ -82,8 +90,6 @@ class Model(Observable):
         self._dosemap = dosemap
         self._dosemap_style = dosemap_style
         
-        
-            
         if walls is not None:
             self.walls.extend(walls)
             
@@ -121,11 +127,6 @@ class Model(Observable):
         
         self.item_selector = ModelItemSelector(self)
         
-
-            
-            
-        
-        
     @property
     def floorplan(self):
         if self._floorplan is None:
@@ -357,37 +358,9 @@ class Model(Observable):
     def sequence_for_item(self, item):
         return self.get_sequence_by_label(item.label)
     
-        
-        
-            
     def match_extent_to_floorplan(self):
         self.dosemap.extent = self.floorplan.extent
         
-   
-    # @property
-    # def shielding_dict(self):
-    #     return {s.name: s for s in self.shieldings}
-        
-    # def get_shielding_by_name(self, shielding_name):
-    #     if len(self.shieldings) == 0: return None
-    #     if shielding_name is None: return None
-        
-    #     shielding = [shielding for shielding in self.shieldings\
-    #                      if shielding.name == shielding_name]
-            
-    #     if len(shielding) > 1:
-    #         msg = f'Multiple shieldings exists with name: {shielding_name}'
-    #         raise KeyError(msg)
-    #     elif len(shielding) == 0:
-    #         msg = f'No shieldings exists with name: {shielding_name}'
-    #         raise KeyError(msg)
-    #     return shielding[0]
-    
-    
-    
-    
-    
-
     @classmethod
     def from_dict(cls, dct): 
         floorplan = Floorplan.from_dict(dct[labels.FLOORPLAN])
@@ -423,6 +396,12 @@ class Model(Observable):
         critical_points = [model_items.CriticalPoint.from_dict(item)\
                            for item in dct[labels.CRITICAL_POINTS]]
             
+        # clearances are not present in older versions (psp files)
+        # ClearanceItemSequence.from_dict resolves compatability and must
+        # be called before loading sources_nm
+        
+        clearances = ClearanceItemSequence.from_dict(dct)
+        
             
         sources_ct = [model_items.SourceCT.from_dict(item)\
                       for item in dct[labels.SOURCES_CT]]
@@ -436,10 +415,7 @@ class Model(Observable):
         
             
             
-        if labels.CLEARANCE in  dct.keys():
-            clearances = ClearanceItemSequence.from_dict(dct)
-        else:
-            clearances = None
+       
             
             
         
@@ -465,16 +441,16 @@ class Model(Observable):
         
     
         
-        walls           = [wall.to_dict() for wall in self.walls]        
-        critical_points = [item.to_dict() for item in self.critical_points]        
-        sources_ct      = [source.to_dict() for source in self.sources_ct]
-        sources_nm      = [source.to_dict() for source in self.sources_nm]
-        sources_xray    = [source.to_dict() for source in self.sources_xray]
+        walls           = [wall.to_dict()   for wall    in self.walls]        
+        critical_points = [item.to_dict()   for item    in self.critical_points]        
+        sources_ct      = [source.to_dict() for source  in self.sources_ct]
+        sources_nm      = [source.to_dict() for source  in self.sources_nm]
+        sources_xray    = [source.to_dict() for source  in self.sources_xray]
         
         # remove the default / empty item
-        shieldings      = [item.to_dict() for item in self.shieldings[1:]]
-        clearances      = [item.to_dict() for item in self.clearances[1:]]
-        materials       = [item.to_dict() for item in self.materials[1:]]
+        shieldings      = [item.to_dict()   for item    in self.shieldings[1:]]
+        clearances      = [item.to_dict()   for item    in self.clearances[1:]]
+        materials       = [item.to_dict()   for item    in self.materials[1:]]
         
             
         return {labels.FLOORPLAN:           floorplan,
@@ -494,7 +470,7 @@ class Model(Observable):
 
     def save_to_project_file(self, filename):    
         file, ext = os.path.splitext(filename)
-        if ext.lower() == '.psp':
+        if ext.lower() != '.zip':
             filename = file + '.zip' # force saving old projects to ext zip
         
         self.filename = filename
@@ -508,9 +484,9 @@ class Model(Observable):
         temp_img = io.BytesIO()
         imageio.imwrite(temp_img, image, format=".png")
 
-        zf = zipfile.ZipFile(filename, "w")
-        zf.writestr("project.yml", temp_yml.getvalue())
-        zf.writestr("floorplan.png", temp_img.getvalue())
+        with zipfile.ZipFile(filename, "w") as zf:
+            zf.writestr("project.yml", temp_yml.getvalue())
+            zf.writestr("floorplan.png", temp_img.getvalue())
     
     
     @classmethod
@@ -529,20 +505,36 @@ class Model(Observable):
         
         except zipfile.BadZipFile:
             ### For backwards compatibility:
+            success = False
             if filename.endswith(".psp"):
-                with open(filename, 'rb') as fp:
-                    dct = pickle.load(fp)
-                image = dct.pop("IMAGE_DATA")   
-                dct[labels.FLOORPLAN][labels.IMAGE] = image
-                dct[labels.FLOORPLAN].pop('Filename', None)
-                model = cls.from_dict(dct)
+                try:
+                    model = cls.load_old_psp_file(filename)
+                    success = True
+                except:
+                    pass
                 
-            else:
+            if not success:
                 # IOError is picked up by GUI to show error dlg
                 raise IOError(f'Could not read {filename}')
     
         model.filename = filename
         return model
+    
+    @classmethod
+    def load_old_psp_file(cls, filename):
+        ### For backwards compatibility:
+        with open(filename, 'rb') as fp:
+            dct = pickle.load(fp)
+        
+        image = dct.pop("IMAGE_DATA")   
+        dct[labels.FLOORPLAN][labels.IMAGE] = image
+        dct[labels.FLOORPLAN].pop('Filename', None)
+        
+        # ugly code to remove invalid key from older versions
+        dct[labels.FLOORPLAN][labels.GEOMETRY].pop(False, False)
+        
+        return cls.from_dict(dct)
+        
 
 class ModelItemSelector(Observable):
     model = None
@@ -619,12 +611,12 @@ class ModelItemSelector(Observable):
         return self._selected_item
 
 
-
 if __name__ == "__main__":
     # import yaml
     # with open('test_model_in.yml') as file:
     #     dct = yaml.safe_load(file)
     # model = Model.from_dict(dct)
     file = '../example_projects/LargeProject/project.zip'
+    # file = 'C:/Users/757021/git/pyrateshield/example_projects/Nucleaire Geneeskunde.psp'
     model = Model.load_from_project_file(file)
     
